#******************************************************************************************************************************************#
#******************************************************************************************************************************************#
#*************************Copyright (c) 2014 Alberto Cassese, Marina Vannucci, Michele Guindani & Mahlet G. Tadesse************************#
#******************************************************************************************************************************************#
#******************************************************************************************************************************************#

############################################################################################################################################
###################################################################INPUTS################################################################### 
############################################################################################################################################                                                                                                              
#Data--------------------------------------------------------------------------------------------------------------------------------------#                                                                                                                                      
#Y_                                    , Matrix of mRNA data                                                                               #
#X_                                    , Matrix of CGH data                                                                                #
#distance_                             , Vector of distance between adjacent CGH probes                                                    #
#disfix_                               , Total length of the DNA fragment                                                                  #
#Starting Values---------------------------------------------------------------------------------------------------------------------------#                                                                                                                           
#xi_                                   , Starting matrix of copy number states                                                             #
#R_                                    , Starting matrix of associations                                                                   #
#tran_                                 , Starting transition matrix                                                                        #
#mu_                                   , Starting vector of state specific mean                                                            #
#sigma_                                , Starting vector of state specific sd                                                              #
#Hyperparameters---------------------------------------------------------------------------------------------------------------------------#                                                                                                                           
#cmu_                                  , Shrinkage parameter, intercept                                                                    #
#cbeta_                                , Shrinkage parameter, coefficients                                                                 #
#delta_,d_                             , Parameters of the Invers-Gamma prior on the gene specific variance                                #
#e_,f_                                 , Parameters of the Beta prior on pi_1                                                              #
#alpha_                                , Tuning parameter that regulates our prior knowledge                                               #
#deltak_,tauk_,upp_bounds_,low_bounds_ , Parameters of the truncated normal on the state specific mean (vectors)                           #
#alpha_IG_,beta_IG_,low_IG_            , Parameters of the truncated Gamma on the state specific precision (vectors)                       #
#a_                                    , Parameters of the Dirichlet distribution on the rows of the transition matrix (vector)            #
#Parameters MCMC---------------------------------------------------------------------------------------------------------------------------#                                                                                                                           
#niter_                                , Number of MCMC iterations                                                                         #
#bi_                                   , Number of iterations discarded as burn-in                                                         #
#phi_                                  , Probability of a Add/Delete step                                                                  #
#pR_                                   , Parameter of the Geometric distribution used to choose the genes updated at each iteration        #
#selectioncgh_                         , Lower bound for the number of samples not in neutral state, for considering a CGH in updating R   #
#pXI_                                  , Parameter of the Geometric distribution used to choose the samples updated at each iteration      #
############################################################################################################################################
###################################################################OUTPUTS##################################################################                                                                                                                                   #
############################################################################################################################################
#A list where:                                                                                                                             #
#The elements [1:niter] are vectors indicating the included positions in the R matrix at each iteration,                                   #
#The elements [(niter+1):(2*niter)] are the transition matrices at each iteration,                                                         #
#The elements [(2*niter+1):(3*niter)] are the vectors of state specific mean at each iteration,                                            #
#The elements [(3*niter+1):(4*niter)] are the vectors of state specific standard deviation at each iteration,                              #
#Element [4*niter+1] correspond to the number of iteration where at least one element of the association matrix R has changed              #
#Element [4*niter+2] correspond to the number of iteration where at least one element of the matrix of hidden states xi has changed        #
#Element [4*niter+3] is the number of accepted MH steps for the transition matrix A                                                        #
############################################################################################################################################

path='PathData/'#Put here the path where your data are
load(paste(path,'Scenario1.RData',sep=""))
library (inline)

#Center the data

for (i in 1:g){
   Y[,i]=Y[,i]-mean(Y[,i])
}
Yvec=NULL
for (i in 1:s){
for (j in 1:g){
Yvec=c(Yvec,Y[i,j])
}
}

#Inputs example

phi=0.5
alpha=20
R=rep(0,g*m)
cbeta=10
d=var(Yvec)/100*5
delta=3
e=0.001
f=0.999
deltak=c(-1,0,0.58,1)
tauk=c(1,1,1,2)
a=c(1,1,1,1)
upp_bounds=c(-0.1, 0.1, 0.73, Inf)
low_bounds=c(-Inf, -0.1, 0.1, 0.73) 
low_IG=c(0.41,0.41,0.41,1)
niter=500000
cmu=1/1000000
alpha_IG=c(1,1,1,1)
beta_IG=c(1,1,1,1)
bi=200000
pxi=0.6
pR=0.4
selection.cgh=10
library(msm)
library(gtools)
mu=c(rtnorm(1,mean=deltak[1],sd=tauk[1],lower=low_bounds[1],upper=upp_bounds[1]),rtnorm(1,mean=deltak[2],sd=tauk[2],lower=low_bounds[2],upper=upp_bounds[2]),rtnorm(1,mean=deltak[3],sd=tauk[3],lower=low_bounds[3],upper=upp_bounds[3]),rtnorm(1,mean=deltak[4],sd=tauk[4],lower=low_bounds[4],upper=upp_bounds[4]))
sigma=(rgamma(4,1,1))^(-0.5)
initial.xi=matrix(1,s,m)
initial.xi[which(X>(-0.5))]=2
initial.xi[which(X>0.29)]=3
initial.xi[which(X>0.79)]=4

path='PathCcodes/'#Put here the path where the Ccodes are
transition <- cxxfunction(signature(xi_="matrix"),paste( readLines(paste(path,'count.cpp',sep="")),collapse = "\n"),plugin = "RcppArmadillo" )

tran=transition(initial.xi)
tran[1,]=tran[1,]/sum(tran[1,])
tran[2,]=tran[2,]/sum(tran[2,])
tran[3,]=tran[3,]/sum(tran[3,])
tran[4,]=tran[4,]/sum(tran[4,])

fx <- cxxfunction(signature(Y_="matrix",X_="matrix",distance_="numeric",disfix_="double",xi_="matrix",R_="numeric",tran_="matrix",mu_="numeric",sigma_="numeric",cmu_="double",cbeta_="double",delta_="double",d_="double",e_="double",f_="double",alpha_="double",deltak_="numeric",tauk_="numeric",upp_bounds_="numeric",low_bounds_="numeric",alpha_IG_="numeric",beta_IG_="numeric",low_IG_="numeric",a_="numeric",niter_="integer",bi_="integer",phi_="double",pR_="double",selectioncgh_="integer",pXI_="double"),paste( readLines(paste(path,"main.cpp",sep="")),collapse = "\n"),plugin = "RcppArmadillo",incl=paste(readLines(paste(path,"include.cpp",sep="")),collapse = "\n")  )
#One of the following can be used
#res=fx(Y_=Y,X_=X,distance_=distance,disfix_=disfix,xi_=initial.xi,R_=R,tran_=tran,mu_=mu,sigma_=sigma,cmu_=cmu,cbeta_=cbeta,delta_=delta,d_=d,e_=e,f_=f,alpha_=alpha,deltak_=deltak,tauk_=tauk,upp_bounds_=upp_bounds,low_bounds_=low_bounds,alpha_IG_=alpha_IG,beta_IG_=beta_IG,low_IG_=low_IG,a_=a,niter_=niter,bi_=bi,phi_=phi,pR_=pR,selectioncgh_=selection.cgh,pXI_=pxi)
res=fx(Y,X,distance,disfix,initial.xi,R,tran,mu,sigma,cmu,cbeta,delta,d,e,f,alpha,deltak,tauk,upp_bounds,low_bounds,alpha_IG,beta_IG,low_IG,a,niter,bi,phi,pR,selection.cgh,pxi)

#Posteprocessing

#Posterior Probabilities of inclusion
GammaD=res[1:niter]

sss = length(GammaD)
GammaBI=GammaD[(bi+1):sss]
b = round(length(GammaBI)/50)
aa = 1
lenMar=m*g

freTot = rep(0,lenMar)
for (j in 1:50){
	bb = b*j
	if (j==50){bb=length(GammaBI)}
	FreqUni = NULL
	for (i in aa:bb){
		FreqUni = c(FreqUni, (as.numeric(GammaBI[[i]])+1))
	}
	fre = tabulate(FreqUni)
	fsize = length(fre)
	ma = (lenMar) - fsize
	Z = rep(0,ma)
	fre = c(fre,Z)
	freTot = fre+freTot
	aa = bb+1
}
FreqMat = NULL
for (i in 1:g){
	FreqMat = rbind(FreqMat,freTot[(1+m*(i-1)):(m*i)])
}
FreqMat=FreqMat/(niter-bi)

#Estimated transition matrix
GammaD=res[(niter+1):(2*niter)]

Amean=matrix(0,nrow=4,ncol=4)

for(i in bi:niter){
temp=GammaD[[i]]
Amean=Amean+temp
}
Amean=Amean/(niter-bi)

#Estimated state specific mean
GammaD=res[(2*niter+1):(3*niter)]

mumean=rep(0,4)

for(i in bi:niter){
temp=GammaD[[i]]
mumean=mumean+temp
}
mumean=mumean/(niter-bi)

#Estimated state specific sd
GammaD=res[(3*niter+1):(4*niter)]

sigmamean=rep(0,4)

for(i in bi:niter){
temp=GammaD[[i]]
sigmamean=sigmamean+temp
}
sigmamean=sigmamean/(niter-bi)

#Save data
path="PathResults/"#Put here the path where you want to save the results
save.image(paste(path,'Results.RData',sep=""))
